@@ -10,10 +10,7 @@
 namespace PHPUnit\Framework\MockObject\Generator;
 
 use const PHP_EOL;
-use const PHP_MAJOR_VERSION;
-use const PHP_MINOR_VERSION;
-use const PREG_OFFSET_CAPTURE;
-use const WSDL_CACHE_NONE;
+use const PHP_VERSION;
 use function array_merge;
 use function array_pop;
 use function array_unique;
@@ -21,44 +18,28 @@ use function assert;
 use function class_exists;
 use function count;
 use function explode;
-use function extension_loaded;
 use function implode;
 use function in_array;
 use function interface_exists;
 use function is_array;
-use function is_object;
 use function md5;
-use function method_exists;
 use function mt_rand;
 use function preg_match;
-use function preg_match_all;
-use function range;
 use function serialize;
 use function sort;
 use function sprintf;
-use function str_contains;
-use function str_replace;
-use function strlen;
-use function strpos;
 use function substr;
 use function trait_exists;
+use function version_compare;
 use Exception;
 use Iterator;
 use IteratorAggregate;
-use PHPUnit\Event\Code\NoTestCaseObjectOnCallStackException;
-use PHPUnit\Event\Code\TestMethodBuilder;
-use PHPUnit\Event\Facade as EventFacade;
-use PHPUnit\Framework\InvalidArgumentException;
 use PHPUnit\Framework\MockObject\ConfigurableMethod;
 use PHPUnit\Framework\MockObject\DoubledCloneMethod;
-use PHPUnit\Framework\MockObject\ErrorCloneMethod;
-use PHPUnit\Framework\MockObject\GeneratedAsMockObject;
-use PHPUnit\Framework\MockObject\GeneratedAsTestStub;
 use PHPUnit\Framework\MockObject\Method;
 use PHPUnit\Framework\MockObject\MockObject;
 use PHPUnit\Framework\MockObject\MockObjectApi;
 use PHPUnit\Framework\MockObject\MockObjectInternal;
-use PHPUnit\Framework\MockObject\MutableStubApi;
 use PHPUnit\Framework\MockObject\ProxiedCloneMethod;
 use PHPUnit\Framework\MockObject\Stub;
 use PHPUnit\Framework\MockObject\StubApi;
@@ -68,11 +49,8 @@ use PropertyHookType;
 use ReflectionClass;
 use ReflectionMethod;
 use ReflectionObject;
-use ReflectionProperty;
 use SebastianBergmann\Type\ReflectionMapper;
 use SebastianBergmann\Type\Type;
-use SoapClient;
-use SoapFault;
 use Throwable;
 use Traversable;
 
@@ -86,78 +64,53 @@ final class Generator
     use TemplateLoader;
 
     /**
-     * @var array<non-empty-string, true>
+     * @var null|non-empty-array<non-empty-string, true>
      */
-    private const EXCLUDED_METHOD_NAMES = [
-        '__CLASS__'       => true,
-        '__DIR__'         => true,
-        '__FILE__'        => true,
-        '__FUNCTION__'    => true,
-        '__LINE__'        => true,
-        '__METHOD__'      => true,
-        '__NAMESPACE__'   => true,
-        '__TRAIT__'       => true,
-        '__clone'         => true,
-        '__halt_compiler' => true,
-    ];
+    private static ?array $excludedMethodNames = null;
 
     /**
-     * @var array<non-empty-string, MockClass>
+     * @var array<non-empty-string, DoubledClass>
      */
     private static array $cache = [];
 
     /**
      * Returns a test double for the specified class.
      *
+     * @param class-string            $type
      * @param ?list<non-empty-string> $methods
-     * @param list<mixed>             $arguments
+     * @param array<mixed>            $arguments
      *
      * @throws ClassIsEnumerationException
      * @throws ClassIsFinalException
      * @throws DuplicateMethodException
      * @throws InvalidMethodNameException
      * @throws NameAlreadyInUseException
-     * @throws OriginalConstructorInvocationRequiredException
      * @throws ReflectionException
      * @throws RuntimeException
      * @throws UnknownTypeException
      */
-    public function testDouble(string $type, bool $mockObject, bool $markAsMockObject, ?array $methods = [], array $arguments = [], string $mockClassName = '', bool $callOriginalConstructor = true, bool $callOriginalClone = true, bool $callAutoload = true, bool $cloneArguments = true, bool $callOriginalMethods = false, ?object $proxyTarget = null, bool $allowMockingUnknownTypes = true, bool $returnValueGeneration = true): MockObject|Stub
+    public function testDouble(string $type, bool $mockObject, ?array $methods = [], array $arguments = [], string $mockClassName = '', bool $callOriginalConstructor = true, bool $callOriginalClone = true, bool $returnValueGeneration = true): MockObject|Stub
     {
         if ($type === Traversable::class) {
             $type = Iterator::class;
         }
 
-        if (!$allowMockingUnknownTypes) {
-            $this->ensureKnownType($type, $callAutoload);
-        }
-
+        $this->ensureKnownType($type);
         $this->ensureValidMethods($methods);
         $this->ensureNameForTestDoubleClassIsAvailable($mockClassName);
 
-        if (!$callOriginalConstructor && $callOriginalMethods) {
-            throw new OriginalConstructorInvocationRequiredException;
-        }
-
         $mock = $this->generate(
             $type,
             $mockObject,
-            $markAsMockObject,
             $methods,
             $mockClassName,
             $callOriginalClone,
-            $callAutoload,
-            $cloneArguments,
-            $callOriginalMethods,
         );
 
-        $object = $this->getObject(
+        $object = $this->instantiate(
             $mock,
-            $type,
             $callOriginalConstructor,
             $arguments,
-            $callOriginalMethods,
-            $proxyTarget,
             $returnValueGeneration,
         );
 
@@ -176,17 +129,17 @@ final class Generator
      * @param list<class-string> $interfaces
      *
      * @throws RuntimeException
-     * @throws UnknownTypeException
+     * @throws UnknownInterfaceException
      */
-    public function testDoubleForInterfaceIntersection(array $interfaces, bool $mockObject, bool $callAutoload = true, bool $returnValueGeneration = true): MockObject|Stub
+    public function testDoubleForInterfaceIntersection(array $interfaces, bool $mockObject, bool $returnValueGeneration = true): MockObject|Stub
     {
         if (count($interfaces) < 2) {
             throw new RuntimeException('At least two interfaces must be specified');
         }
 
         foreach ($interfaces as $interface) {
-            if (!interface_exists($interface, $callAutoload)) {
-                throw new UnknownTypeException($interface);
+            if (!interface_exists($interface)) {
+                throw new UnknownInterfaceException($interface);
             }
         }
 
@@ -230,174 +183,17 @@ final class Generator
 
         eval($template->render());
 
+        assert(interface_exists($intersectionName));
+
         return $this->testDouble(
             $intersectionName,
             $mockObject,
-            $mockObject,
             returnValueGeneration: $returnValueGeneration,
         );
     }
 
     /**
-     * Returns a mock object for the specified abstract class with all abstract
-     * methods of the class mocked.
-     *
-     * Concrete methods to mock can be specified with the $mockedMethods parameter.
-     *
-     * @param list<mixed>             $arguments
-     * @param ?list<non-empty-string> $mockedMethods
-     *
-     * @throws ClassIsEnumerationException
-     * @throws ClassIsFinalException
-     * @throws DuplicateMethodException
-     * @throws InvalidArgumentException
-     * @throws InvalidMethodNameException
-     * @throws NameAlreadyInUseException
-     * @throws OriginalConstructorInvocationRequiredException
-     * @throws ReflectionException
-     * @throws RuntimeException
-     * @throws UnknownClassException
-     * @throws UnknownTypeException
-     *
-     * @deprecated https://github.com/sebastianbergmann/phpunit/issues/5241
-     */
-    public function mockObjectForAbstractClass(string $originalClassName, array $arguments = [], string $mockClassName = '', bool $callOriginalConstructor = true, bool $callOriginalClone = true, bool $callAutoload = true, ?array $mockedMethods = null, bool $cloneArguments = true): MockObject
-    {
-        if (class_exists($originalClassName, $callAutoload) ||
-            interface_exists($originalClassName, $callAutoload)) {
-            $reflector = $this->reflectClass($originalClassName);
-            $methods   = $mockedMethods;
-
-            foreach ($reflector->getMethods() as $method) {
-                if ($method->isAbstract() && !in_array($method->getName(), $methods ?? [], true)) {
-                    $methods[] = $method->getName();
-                }
-            }
-
-            if (empty($methods)) {
-                $methods = null;
-            }
-
-            $mockObject = $this->testDouble(
-                $originalClassName,
-                true,
-                true,
-                $methods,
-                $arguments,
-                $mockClassName,
-                $callOriginalConstructor,
-                $callOriginalClone,
-                $callAutoload,
-                $cloneArguments,
-            );
-
-            assert($mockObject instanceof $originalClassName);
-            assert($mockObject instanceof MockObject);
-
-            return $mockObject;
-        }
-
-        throw new UnknownClassException($originalClassName);
-    }
-
-    /**
-     * Returns a mock object for the specified trait with all abstract methods
-     * of the trait mocked. Concrete methods to mock can be specified with the
-     * `$mockedMethods` parameter.
-     *
-     * @param trait-string            $traitName
-     * @param list<mixed>             $arguments
-     * @param ?list<non-empty-string> $mockedMethods
-     *
-     * @throws ClassIsEnumerationException
-     * @throws ClassIsFinalException
-     * @throws DuplicateMethodException
-     * @throws InvalidArgumentException
-     * @throws InvalidMethodNameException
-     * @throws NameAlreadyInUseException
-     * @throws OriginalConstructorInvocationRequiredException
-     * @throws ReflectionException
-     * @throws RuntimeException
-     * @throws UnknownClassException
-     * @throws UnknownTraitException
-     * @throws UnknownTypeException
-     *
-     * @deprecated https://github.com/sebastianbergmann/phpunit/issues/5243
-     */
-    public function mockObjectForTrait(string $traitName, array $arguments = [], string $mockClassName = '', bool $callOriginalConstructor = true, bool $callOriginalClone = true, bool $callAutoload = true, ?array $mockedMethods = null, bool $cloneArguments = true): MockObject
-    {
-        if (!trait_exists($traitName, $callAutoload)) {
-            throw new UnknownTraitException($traitName);
-        }
-
-        $className = $this->generateClassName(
-            $traitName,
-            '',
-            'Trait_',
-        );
-
-        $classTemplate = $this->loadTemplate('trait_class.tpl');
-
-        $classTemplate->setVar(
-            [
-                'prologue'   => 'abstract ',
-                'class_name' => $className['className'],
-                'trait_name' => $traitName,
-            ],
-        );
-
-        $mockTrait = new MockTrait($classTemplate->render(), $className['className']);
-        $mockTrait->generate();
-
-        return $this->mockObjectForAbstractClass($className['className'], $arguments, $mockClassName, $callOriginalConstructor, $callOriginalClone, $callAutoload, $mockedMethods, $cloneArguments);
-    }
-
-    /**
-     * Returns an object for the specified trait.
-     *
-     * @param trait-string $traitName
-     * @param list<mixed>  $arguments
-     *
-     * @throws ReflectionException
-     * @throws RuntimeException
-     * @throws UnknownTraitException
-     *
-     * @deprecated https://github.com/sebastianbergmann/phpunit/issues/5244
-     */
-    public function objectForTrait(string $traitName, string $traitClassName = '', bool $callAutoload = true, bool $callOriginalConstructor = false, array $arguments = []): object
-    {
-        if (!trait_exists($traitName, $callAutoload)) {
-            throw new UnknownTraitException($traitName);
-        }
-
-        $className = $this->generateClassName(
-            $traitName,
-            $traitClassName,
-            'Trait_',
-        );
-
-        $classTemplate = $this->loadTemplate('trait_class.tpl');
-
-        $classTemplate->setVar(
-            [
-                'prologue'   => '',
-                'class_name' => $className['className'],
-                'trait_name' => $traitName,
-            ],
-        );
-
-        return $this->getObject(
-            new MockTrait(
-                $classTemplate->render(),
-                $className['className'],
-            ),
-            '',
-            $callOriginalConstructor,
-            $arguments,
-        );
-    }
-
-    /**
+     * @param class-string            $type
      * @param ?list<non-empty-string> $methods
      *
      * @throws ClassIsEnumerationException
@@ -409,43 +205,32 @@ final class Generator
      *
      * @see https://github.com/sebastianbergmann/phpunit/issues/5476
      */
-    public function generate(string $type, bool $mockObject, bool $markAsMockObject, ?array $methods = null, string $mockClassName = '', bool $callOriginalClone = true, bool $callAutoload = true, bool $cloneArguments = true, bool $callOriginalMethods = false): MockClass
+    public function generate(string $type, bool $mockObject, ?array $methods = null, string $mockClassName = '', bool $callOriginalClone = true): DoubledClass
     {
         if ($mockClassName !== '') {
             return $this->generateCodeForTestDoubleClass(
                 $type,
                 $mockObject,
-                $markAsMockObject,
                 $methods,
                 $mockClassName,
                 $callOriginalClone,
-                $callAutoload,
-                $cloneArguments,
-                $callOriginalMethods,
             );
         }
 
         $key = md5(
             $type .
             ($mockObject ? 'MockObject' : 'TestStub') .
-            ($markAsMockObject ? 'MockObject' : 'TestStub') .
             serialize($methods) .
-            serialize($callOriginalClone) .
-            serialize($cloneArguments) .
-            serialize($callOriginalMethods),
+            serialize($callOriginalClone),
         );
 
         if (!isset(self::$cache[$key])) {
             self::$cache[$key] = $this->generateCodeForTestDoubleClass(
                 $type,
                 $mockObject,
-                $markAsMockObject,
                 $methods,
                 $mockClassName,
                 $callOriginalClone,
-                $callAutoload,
-                $cloneArguments,
-                $callOriginalMethods,
             );
         }
 
@@ -453,120 +238,20 @@ final class Generator
     }
 
     /**
-     * @param non-empty-string       $wsdlFile
-     * @param class-string           $className
-     * @param list<non-empty-string> $methods
-     * @param array<mixed>           $options
+     * @param class-string $className
      *
-     * @throws RuntimeException
-     * @throws SoapExtensionNotAvailableException
-     *
-     * @deprecated https://github.com/sebastianbergmann/phpunit/issues/5242
-     */
-    public function generateClassFromWsdl(string $wsdlFile, string $className, array $methods = [], array $options = []): string
-    {
-        if (!extension_loaded('soap')) {
-            throw new SoapExtensionNotAvailableException;
-        }
-
-        $options['cache_wsdl'] = WSDL_CACHE_NONE;
-
-        try {
-            $client   = new SoapClient($wsdlFile, $options);
-            $_methods = array_unique($client->__getFunctions() ?? []);
-
-            unset($client);
-        } catch (SoapFault $e) {
-            throw new RuntimeException(
-                $e->getMessage(),
-                $e->getCode(),
-                $e,
-            );
-        }
-
-        sort($_methods);
-
-        $methodTemplate = $this->loadTemplate('wsdl_method.tpl');
-        $methodsBuffer  = '';
-
-        foreach ($_methods as $method) {
-            preg_match_all('/[a-zA-Z_\x7f-\xff][a-zA-Z0-9_\x7f-\xff]*\(/', $method, $matches, PREG_OFFSET_CAPTURE);
-
-            $lastFunction = array_pop($matches[0]);
-            $nameStart    = $lastFunction[1];
-            $nameEnd      = $nameStart + strlen($lastFunction[0]) - 1;
-            $name         = str_replace('(', '', $lastFunction[0]);
-
-            if (empty($methods) || in_array($name, $methods, true)) {
-                $arguments = explode(
-                    ',',
-                    str_replace(')', '', substr($method, $nameEnd + 1)),
-                );
-
-                foreach (range(0, count($arguments) - 1) as $i) {
-                    $parameterStart = strpos($arguments[$i], '$');
-
-                    if (!$parameterStart) {
-                        continue;
-                    }
-
-                    $arguments[$i] = substr($arguments[$i], $parameterStart);
-                }
-
-                $methodTemplate->setVar(
-                    [
-                        'method_name' => $name,
-                        'arguments'   => implode(', ', $arguments),
-                    ],
-                );
-
-                $methodsBuffer .= $methodTemplate->render();
-            }
-        }
-
-        $optionsBuffer = '[';
-
-        foreach ($options as $key => $value) {
-            $optionsBuffer .= $key . ' => ' . $value;
-        }
-
-        $optionsBuffer .= ']';
-
-        $classTemplate = $this->loadTemplate('wsdl_class.tpl');
-        $namespace     = '';
-
-        if (str_contains($className, '\\')) {
-            $parts     = explode('\\', $className);
-            $className = array_pop($parts);
-            $namespace = 'namespace ' . implode('\\', $parts) . ';' . "\n\n";
-        }
-
-        $classTemplate->setVar(
-            [
-                'namespace'  => $namespace,
-                'class_name' => $className,
-                'wsdl'       => $wsdlFile,
-                'options'    => $optionsBuffer,
-                'methods'    => $methodsBuffer,
-            ],
-        );
-
-        return $classTemplate->render();
-    }
-
-    /**
      * @throws ReflectionException
      *
-     * @return list<MockMethod>
+     * @return list<DoubledMethod>
      */
-    public function mockClassMethods(string $className, bool $callOriginalMethods, bool $cloneArguments): array
+    private function mockClassMethods(string $className): array
     {
         $class   = $this->reflectClass($className);
         $methods = [];
 
         foreach ($class->getMethods() as $method) {
             if (($method->isPublic() || $method->isAbstract()) && $this->canMethodBeDoubled($method)) {
-                $methods[] = MockMethod::fromReflection($method, $callOriginalMethods, $cloneArguments);
+                $methods[] = DoubledMethod::fromReflection($method);
             }
         }
 
@@ -602,7 +287,7 @@ final class Generator
      * @throws ReflectionException
      * @throws RuntimeException
      */
-    private function getObject(MockType $mockClass, string $type = '', bool $callOriginalConstructor = false, array $arguments = [], bool $callOriginalMethods = false, ?object $proxyTarget = null, bool $returnValueGeneration = true): object
+    private function instantiate(DoubledClass $mockClass, bool $callOriginalConstructor = false, array $arguments = [], bool $returnValueGeneration = true): object
     {
         $className = $mockClass->generate();
 
@@ -620,19 +305,13 @@ final class Generator
 
         $reflector = new ReflectionObject($object);
 
-        if ($object instanceof StubInternal && $mockClass instanceof MockClass) {
-            /**
-             * @noinspection PhpUnhandledExceptionInspection
-             */
-            $reflector->getProperty('__phpunit_state')->setValue(
-                $object,
-                new TestDoubleState($mockClass->configurableMethods(), $returnValueGeneration),
-            );
-
-            if ($callOriginalMethods) {
-                $this->instantiateProxyTarget($proxyTarget, $object, $type, $arguments);
-            }
-        }
+        /**
+         * @noinspection PhpUnhandledExceptionInspection
+         */
+        $reflector->getProperty('__phpunit_state')->setValue(
+            $object,
+            new TestDoubleState($mockClass->configurableMethods(), $returnValueGeneration),
+        );
 
         if ($callOriginalConstructor && $reflector->getConstructor() !== null) {
             try {
@@ -652,14 +331,16 @@ final class Generator
     }
 
     /**
+     * @param class-string            $type
      * @param ?list<non-empty-string> $explicitMethods
      *
      * @throws ClassIsEnumerationException
      * @throws ClassIsFinalException
+     * @throws MethodNamedMethodException
      * @throws ReflectionException
      * @throws RuntimeException
      */
-    private function generateCodeForTestDoubleClass(string $type, bool $mockObject, bool $markAsMockObject, ?array $explicitMethods, string $mockClassName, bool $callOriginalClone, bool $callAutoload, bool $cloneArguments, bool $callOriginalMethods): MockClass
+    private function generateCodeForTestDoubleClass(string $type, bool $mockObject, ?array $explicitMethods, string $mockClassName, bool $callOriginalClone): DoubledClass
     {
         $classTemplate         = $this->loadTemplate('test_double_class.tpl');
         $additionalInterfaces  = [];
@@ -668,8 +349,7 @@ final class Generator
         $isClass               = false;
         $isReadonly            = false;
         $isInterface           = false;
-        $class                 = null;
-        $mockMethods           = new MockMethodSet;
+        $mockMethods           = new DoubledMethodSet;
         $testDoubleClassPrefix = $mockObject ? 'MockObject_' : 'TestStub_';
 
         $_mockClassName = $this->generateClassName(
@@ -678,104 +358,90 @@ final class Generator
             $testDoubleClassPrefix,
         );
 
-        if (class_exists($_mockClassName['fullClassName'], $callAutoload)) {
+        if (class_exists($_mockClassName['fullClassName'])) {
             $isClass = true;
-        } elseif (interface_exists($_mockClassName['fullClassName'], $callAutoload)) {
+        } elseif (interface_exists($_mockClassName['fullClassName'])) {
             $isInterface = true;
         }
 
-        if (!$isClass && !$isInterface) {
-            $prologue = 'class ' . $_mockClassName['originalClassName'] . "\n{\n}\n\n";
+        $class = $this->reflectClass($_mockClassName['fullClassName']);
 
-            if (!empty($_mockClassName['namespaceName'])) {
-                $prologue = 'namespace ' . $_mockClassName['namespaceName'] .
-                            " {\n\n" . $prologue . "}\n\n" .
-                            "namespace {\n\n";
+        if ($class->isEnum()) {
+            throw new ClassIsEnumerationException($_mockClassName['fullClassName']);
+        }
 
-                $epilogue = "\n\n}";
-            }
+        if ($class->isFinal()) {
+            throw new ClassIsFinalException($_mockClassName['fullClassName']);
+        }
 
-            $doubledCloneMethod = true;
-        } else {
-            $class = $this->reflectClass($_mockClassName['fullClassName']);
+        if ($class->isReadOnly()) {
+            $isReadonly = true;
+        }
 
-            if ($class->isEnum()) {
-                throw new ClassIsEnumerationException($_mockClassName['fullClassName']);
-            }
+        // @see https://github.com/sebastianbergmann/phpunit/issues/2995
+        if ($isInterface && $class->implementsInterface(Throwable::class)) {
+            $actualClassName        = Exception::class;
+            $additionalInterfaces[] = $class->getName();
+            $isInterface            = false;
+            $class                  = $this->reflectClass($actualClassName);
 
-            if ($class->isFinal()) {
-                throw new ClassIsFinalException($_mockClassName['fullClassName']);
-            }
+            foreach ($this->userDefinedInterfaceMethods($_mockClassName['fullClassName']) as $method) {
+                $methodName = $method->getName();
 
-            if ($class->isReadOnly()) {
-                $isReadonly = true;
-            }
+                if ($class->hasMethod($methodName)) {
+                    $classMethod = $class->getMethod($methodName);
 
-            // @see https://github.com/sebastianbergmann/phpunit/issues/2995
-            if ($isInterface && $class->implementsInterface(Throwable::class)) {
-                $actualClassName        = Exception::class;
-                $additionalInterfaces[] = $class->getName();
-                $isInterface            = false;
-                $class                  = $this->reflectClass($actualClassName);
-
-                foreach ($this->userDefinedInterfaceMethods($_mockClassName['fullClassName']) as $method) {
-                    $methodName = $method->getName();
-
-                    if ($class->hasMethod($methodName)) {
-                        $classMethod = $class->getMethod($methodName);
-
-                        if (!$this->canMethodBeDoubled($classMethod)) {
-                            continue;
-                        }
+                    if (!$this->canMethodBeDoubled($classMethod)) {
+                        continue;
                     }
-
-                    $mockMethods->addMethods(
-                        MockMethod::fromReflection($method, $callOriginalMethods, $cloneArguments),
-                    );
                 }
 
-                $_mockClassName = $this->generateClassName(
-                    $actualClassName,
-                    $_mockClassName['className'],
-                    $testDoubleClassPrefix,
-                );
-            }
-
-            // @see https://github.com/sebastianbergmann/phpunit-mock-objects/issues/103
-            if ($isInterface && $class->implementsInterface(Traversable::class) &&
-                !$class->implementsInterface(Iterator::class) &&
-                !$class->implementsInterface(IteratorAggregate::class)) {
-                $additionalInterfaces[] = Iterator::class;
-
                 $mockMethods->addMethods(
-                    ...$this->mockClassMethods(Iterator::class, $callOriginalMethods, $cloneArguments),
+                    DoubledMethod::fromReflection($method),
                 );
             }
 
-            if ($class->hasMethod('__clone')) {
-                $cloneMethod = $class->getMethod('__clone');
+            $_mockClassName = $this->generateClassName(
+                $actualClassName,
+                $_mockClassName['className'],
+                $testDoubleClassPrefix,
+            );
+        }
 
-                if (!$cloneMethod->isFinal()) {
-                    if ($callOriginalClone && !$isInterface) {
-                        $proxiedCloneMethod = true;
-                    } else {
-                        $doubledCloneMethod = true;
-                    }
+        // @see https://github.com/sebastianbergmann/phpunit-mock-objects/issues/103
+        if ($isInterface && $class->implementsInterface(Traversable::class) &&
+            !$class->implementsInterface(Iterator::class) &&
+            !$class->implementsInterface(IteratorAggregate::class)) {
+            $additionalInterfaces[] = Iterator::class;
+
+            $mockMethods->addMethods(
+                ...$this->mockClassMethods(Iterator::class),
+            );
+        }
+
+        if ($class->hasMethod('__clone')) {
+            $cloneMethod = $class->getMethod('__clone');
+
+            if (!$cloneMethod->isFinal()) {
+                if ($callOriginalClone && !$isInterface) {
+                    $proxiedCloneMethod = true;
+                } else {
+                    $doubledCloneMethod = true;
                 }
-            } else {
-                $doubledCloneMethod = true;
             }
+        } else {
+            $doubledCloneMethod = true;
         }
 
         if ($isClass && $explicitMethods === []) {
             $mockMethods->addMethods(
-                ...$this->mockClassMethods($_mockClassName['fullClassName'], $callOriginalMethods, $cloneArguments),
+                ...$this->mockClassMethods($_mockClassName['fullClassName']),
             );
         }
 
         if ($isInterface && ($explicitMethods === [] || $explicitMethods === null)) {
             $mockMethods->addMethods(
-                ...$this->interfaceMethods($_mockClassName['fullClassName'], $cloneArguments),
+                ...$this->interfaceMethods($_mockClassName['fullClassName']),
             );
         }
 
@@ -786,15 +452,14 @@ final class Generator
 
                     if ($this->canMethodBeDoubled($method)) {
                         $mockMethods->addMethods(
-                            MockMethod::fromReflection($method, $callOriginalMethods, $cloneArguments),
+                            DoubledMethod::fromReflection($method),
                         );
                     }
                 } else {
                     $mockMethods->addMethods(
-                        MockMethod::fromName(
+                        DoubledMethod::fromName(
                             $_mockClassName['fullClassName'],
                             $methodName,
-                            $cloneArguments,
                         ),
                     );
                 }
@@ -811,61 +476,22 @@ final class Generator
         }
 
         /** @var trait-string[] $traits */
-        $traits = [];
-
-        /** @phpstan-ignore identical.alwaysTrue */
-        $isPhp82 = PHP_MAJOR_VERSION === 8 && PHP_MINOR_VERSION === 2;
-
-        if (!$isReadonly && $isPhp82) {
-            // @codeCoverageIgnoreStart
-            $traits[] = MutableStubApi::class;
-            // @codeCoverageIgnoreEnd
-        } else {
-            $traits[] = StubApi::class;
-        }
+        $traits = [StubApi::class];
 
         if ($mockObject) {
             $traits[] = MockObjectApi::class;
         }
 
-        if ($markAsMockObject) {
-            $traits[] = GeneratedAsMockObject::class;
-        } else {
-            $traits[] = GeneratedAsTestStub::class;
+        if ($mockMethods->hasMethod('method') || $class->hasMethod('method')) {
+            throw new MethodNamedMethodException;
         }
 
-        if ($mockMethods->hasMethod('method') || (isset($class) && $class->hasMethod('method'))) {
-            $message = sprintf(
-                '%s %s has a method named "method". Doubling %s that have a method named "method" is deprecated. Support for this will be removed in PHPUnit 12.',
-                ($isInterface) ? 'Interface' : 'Class',
-                isset($class) ? $class->getName() : $type,
-                ($isInterface) ? 'interfaces' : 'classes',
-            );
+        $traits[] = Method::class;
 
-            try {
-                EventFacade::emitter()->testTriggeredPhpunitDeprecation(
-                    TestMethodBuilder::fromCallStack(),
-                    $message,
-                );
-            } catch (NoTestCaseObjectOnCallStackException) {
-                EventFacade::emitter()->testRunnerTriggeredDeprecation($message);
-            }
-        }
-
-        if (!$mockMethods->hasMethod('method') && (!isset($class) || !$class->hasMethod('method'))) {
-            $traits[] = Method::class;
-        }
-
-        if ($isPhp82 && $isReadonly) {
-            // @codeCoverageIgnoreStart
-            $traits[] = ErrorCloneMethod::class;
-            // @codeCoverageIgnoreEnd
-        } else {
-            if ($doubledCloneMethod) {
-                $traits[] = DoubledCloneMethod::class;
-            } elseif ($proxiedCloneMethod) {
-                $traits[] = ProxiedCloneMethod::class;
-            }
+        if ($doubledCloneMethod) {
+            $traits[] = DoubledCloneMethod::class;
+        } elseif ($proxiedCloneMethod) {
+            $traits[] = ProxiedCloneMethod::class;
         }
 
         $useStatements = '';
@@ -881,8 +507,6 @@ final class Generator
 
         $classTemplate->setVar(
             [
-                'prologue'          => $prologue ?? '',
-                'epilogue'          => $epilogue ?? '',
                 'class_declaration' => $this->generateTestDoubleClassDeclaration(
                     $mockObject,
                     $_mockClassName,
@@ -900,7 +524,7 @@ final class Generator
             ],
         );
 
-        return new MockClass(
+        return new DoubledClass(
             $classTemplate->render(),
             $_mockClassName['className'],
             $configurableMethods,
@@ -908,7 +532,9 @@ final class Generator
     }
 
     /**
-     * @return array{className: non-empty-string, originalClassName: non-empty-string, fullClassName: non-empty-string, namespaceName: string}
+     * @param class-string $type
+     *
+     * @return array{className: class-string, originalClassName: class-string, fullClassName: class-string, namespaceName: string}
      */
     private function generateClassName(string $type, string $className, string $prefix): array
     {
@@ -972,7 +598,7 @@ final class Generator
             if (!in_array($mockClassName['originalClassName'], $additionalInterfaces, true)) {
                 $buffer .= ', ';
 
-                if (!empty($mockClassName['namespaceName'])) {
+                if ($mockClassName['namespaceName'] !== '') {
                     $buffer .= $mockClassName['namespaceName'] . '\\';
                 }
 
@@ -982,7 +608,7 @@ final class Generator
             $buffer .= sprintf(
                 '%s extends %s%s implements %s',
                 $mockClassName['className'],
-                !empty($mockClassName['namespaceName']) ? $mockClassName['namespaceName'] . '\\' : '',
+                $mockClassName['namespaceName'] !== '' ? $mockClassName['namespaceName'] . '\\' : '',
                 $mockClassName['originalClassName'],
                 $interfaces,
             );
@@ -1014,15 +640,35 @@ final class Generator
 
     private function isMethodNameExcluded(string $name): bool
     {
-        return isset(self::EXCLUDED_METHOD_NAMES[$name]);
+        if (self::$excludedMethodNames === null) {
+            self::$excludedMethodNames = [
+                '__CLASS__'       => true,
+                '__DIR__'         => true,
+                '__FILE__'        => true,
+                '__FUNCTION__'    => true,
+                '__LINE__'        => true,
+                '__METHOD__'      => true,
+                '__NAMESPACE__'   => true,
+                '__TRAIT__'       => true,
+                '__clone'         => true,
+                '__halt_compiler' => true,
+            ];
+
+            if (version_compare(PHP_VERSION, '8.5', '>=')) {
+                self::$excludedMethodNames['__sleep']  = true;
+                self::$excludedMethodNames['__wakeup'] = true;
+            }
+        }
+
+        return isset(self::$excludedMethodNames[$name]);
     }
 
     /**
      * @throws UnknownTypeException
      */
-    private function ensureKnownType(string $type, bool $callAutoload): void
+    private function ensureKnownType(string $type): void
     {
-        if (!class_exists($type, $callAutoload) && !interface_exists($type, $callAutoload)) {
+        if (!class_exists($type) && !interface_exists($type)) {
             throw new UnknownTypeException($type);
         }
     }
@@ -1067,39 +713,6 @@ final class Generator
         }
     }
 
-    /**
-     * @param class-string $type
-     * @param array<mixed> $arguments
-     *
-     * @throws ReflectionException
-     */
-    private function instantiateProxyTarget(?object $proxyTarget, object $object, string $type, array $arguments): void
-    {
-        if (!is_object($proxyTarget)) {
-            assert(class_exists($type));
-
-            if (count($arguments) === 0) {
-                $proxyTarget = new $type;
-            } else {
-                $class = new ReflectionClass($type);
-
-                try {
-                    $proxyTarget = $class->newInstanceArgs($arguments);
-                    // @codeCoverageIgnoreStart
-                } catch (\ReflectionException $e) {
-                    throw new ReflectionException(
-                        $e->getMessage(),
-                        $e->getCode(),
-                        $e,
-                    );
-                }
-                // @codeCoverageIgnoreEnd
-            }
-        }
-
-        $object->__phpunit_state()->setProxyTarget($proxyTarget);
-    }
-
     /**
      * @param class-string $className
      *
@@ -1152,15 +765,15 @@ final class Generator
      *
      * @throws ReflectionException
      *
-     * @return list<MockMethod>
+     * @return list<DoubledMethod>
      */
-    private function interfaceMethods(string $interfaceName, bool $cloneArguments): array
+    private function interfaceMethods(string $interfaceName): array
     {
         $class   = $this->reflectClass($interfaceName);
         $methods = [];
 
         foreach ($class->getMethods() as $method) {
-            $methods[] = MockMethod::fromReflection($method, false, $cloneArguments);
+            $methods[] = DoubledMethod::fromReflection($method);
         }
 
         return $methods;
@@ -1171,7 +784,7 @@ final class Generator
      *
      * @return list<ConfigurableMethod>
      */
-    private function configurableMethods(MockMethodSet $methods, array $propertiesWithHooks): array
+    private function configurableMethods(DoubledMethodSet $methods, array $propertiesWithHooks): array
     {
         $configurable = [];
 
@@ -1220,7 +833,7 @@ final class Generator
      */
     private function properties(?ReflectionClass $class): array
     {
-        if (!method_exists(ReflectionProperty::class, 'isFinal')) {
+        if (version_compare('8.4.1', PHP_VERSION, '>')) {
             // @codeCoverageIgnoreStart
             return [];
             // @codeCoverageIgnoreEnd
@@ -1234,12 +847,6 @@ final class Generator
         $properties = [];
 
         foreach ($class->getProperties() as $property) {
-            assert(method_exists($property, 'getHook'));
-            assert(method_exists($property, 'hasHooks'));
-            assert(method_exists($property, 'hasHook'));
-            assert(method_exists($property, 'isFinal'));
-            assert(class_exists(PropertyHookType::class));
-
             if (!$property->isPublic()) {
                 continue;
             }
