@@ -44,14 +44,4 @@ final readonly class CoversTrait extends Metadata
     {
         return $this->traitName;
     }
-
-    /**
-     * @return trait-string
-     *
-     * @internal This method is not covered by the backward compatibility promise for PHPUnit
-     */
-    public function asStringForCodeUnitMapper(): string
-    {
-        return $this->traitName;
-    }
 }
