@@ -7,9 +7,11 @@
  * For the full copyright and license information, please view the LICENSE
  * file that was distributed with this source code.
  */
+namespace PHPUnit\TestFixture;
+
 use PHPUnit\Framework\TestCase;
 
-class DependencyInputTest extends TestCase
+final class DependencyInputTest extends TestCase
 {
     public function testDependencyInputAsParameter(string $dependencyInput): void
     {
