@@ -9,6 +9,8 @@
  */
 namespace PHPUnit\TestFixture\Event;
 
+use const E_USER_NOTICE;
+use function error_get_last;
 use function trigger_error;
 use PHPUnit\Framework\TestCase;
 
@@ -20,4 +22,11 @@ final class SuppressedUserNoticeTest extends TestCase
 
         @trigger_error('message', E_USER_NOTICE);
     }
+
+    public function testSuppressedUserNoticeErrorGetLast(): void
+    {
+        $this->assertNull(error_get_last());
+        @trigger_error('message', E_USER_NOTICE);
+        $this->assertIsArray(error_get_last());
+    }
 }
