@@ -9,7 +9,10 @@
  */
 namespace PHPUnit\TestFixture;
 
+use PHPUnit\Framework\Attributes\DataProvider;
+use PHPUnit\Framework\ExpectationFailedException;
 use PHPUnit\Framework\TestCase;
+use SebastianBergmann\RecursionContext\InvalidArgumentException;
 use stdClass;
 
 class Issue2137Test extends TestCase
@@ -17,30 +20,28 @@ class Issue2137Test extends TestCase
     public static function provideBrandService()
     {
         return [
-            //[true, true]
+            // [true, true]
             new stdClass, // not valid
         ];
     }
 
     /**
-     * @dataProvider provideBrandService
-     *
-     * @throws \PHPUnit\Framework\ExpectationFailedException
-     * @throws \SebastianBergmann\RecursionContext\InvalidArgumentException
      * @throws Exception
+     * @throws ExpectationFailedException
+     * @throws InvalidArgumentException
      */
+    #[DataProvider('provideBrandService')]
     public function testBrandService($provided, $expected): void
     {
         $this->assertSame($provided, $expected);
     }
 
     /**
-     * @dataProvider provideBrandService
-     *
      * @throws \Exception
-     * @throws \PHPUnit\Framework\ExpectationFailedException
-     * @throws \SebastianBergmann\RecursionContext\InvalidArgumentException
+     * @throws ExpectationFailedException
+     * @throws InvalidArgumentException
      */
+    #[DataProvider('provideBrandService')]
     public function testSomethingElseInvalid($provided, $expected): void
     {
         $this->assertSame($provided, $expected);
