@@ -9,19 +9,14 @@
  */
 namespace PHPUnit\TestFixture;
 
+use PHPUnit\Framework\Attributes\DataProvider;
+use PHPUnit\Framework\Attributes\Depends;
 use PHPUnit\Framework\TestCase;
 use stdClass;
 
 class Issue3156Test extends TestCase
 {
-    public function testConstants(): stdClass
-    {
-        $this->assertStringEndsWith('/', '/');
-
-        return new stdClass;
-    }
-
-    public function dataSelectOperatorsProvider(): array
+    public static function dataSelectOperatorsProvider(): array
     {
         return [
             ['1'],
@@ -29,11 +24,15 @@ class Issue3156Test extends TestCase
         ];
     }
 
-    /**
-     * @depends testConstants
-     *
-     * @dataProvider dataSelectOperatorsProvider
-     */
+    public function testConstants(): stdClass
+    {
+        $this->assertStringEndsWith('/', '/');
+
+        return new stdClass;
+    }
+
+    #[Depends('testConstants')]
+    #[DataProvider('dataSelectOperatorsProvider')]
     public function testDependsRequire(string $val, stdClass $obj): void
     {
         $this->assertStringEndsWith('/', '/');
