@@ -7,6 +7,9 @@
  * For the full copyright and license information, please view the LICENSE
  * file that was distributed with this source code.
  */
+namespace PHPUnit\TestFixture\Issue5288;
+
+use PHPUnit\Framework\Attributes\DataProvider;
 use PHPUnit\Framework\TestCase;
 
 final class Issue5288Test extends TestCase
@@ -16,9 +19,7 @@ final class Issue5288Test extends TestCase
         return [[true]];
     }
 
-    /**
-     * @dataProvider provider()
-     */
+    #[DataProvider('provider')]
     public function testOne(bool $value): void
     {
         $this->assertTrue($value);
