@@ -27,10 +27,10 @@ final class LogicalAndTest extends TestCase
         return [
             [
                 true,
-                'is of type boolean and is true',
+                'is of type bool and is true',
                 '',
                 self::logicalAnd(
-                    self::isType('boolean'),
+                    self::isBool(),
                     self::isTrue(),
                 ),
                 true,
@@ -38,10 +38,21 @@ final class LogicalAndTest extends TestCase
 
             [
                 true,
-                'is of type boolean and ( is true or is false )',
+                'is of type bool and is equal to true',
                 '',
                 self::logicalAnd(
-                    self::isType('boolean'),
+                    self::isBool(),
+                    true,
+                ),
+                true,
+            ],
+
+            [
+                true,
+                'is of type bool and ( is true or is false )',
+                '',
+                self::logicalAnd(
+                    self::isBool(),
                     self::logicalOr(
                         self::isTrue(),
                         self::isFalse(),
@@ -52,10 +63,10 @@ final class LogicalAndTest extends TestCase
 
             [
                 false,
-                'is of type boolean and is true',
-                'Failed asserting that false is of type boolean and is true.',
+                'is of type bool and is true',
+                'Failed asserting that false is of type bool and is true.',
                 self::logicalAnd(
-                    self::isType('boolean'),
+                    self::isBool(),
                     self::isTrue(),
                 ),
                 false,
@@ -63,10 +74,10 @@ final class LogicalAndTest extends TestCase
 
             [
                 false,
-                'is of type boolean and ( is true or is false )',
-                'Failed asserting that \'string\' is of type boolean and ( is true or is false ).',
+                'is of type bool and ( is true or is false )',
+                'Failed asserting that \'string\' is of type bool and ( is true or is false ).',
                 self::logicalAnd(
-                    self::isType('boolean'),
+                    self::isBool(),
                     self::logicalOr(
                         self::isTrue(),
                         self::isFalse(),
@@ -101,8 +112,8 @@ final class LogicalAndTest extends TestCase
     public function testIsCountable(): void
     {
         $constraint = $this->logicalAnd(
-            $this->isType('bool'),
-            $this->isTrue(),
+            $this->isBool(),
+            true,
         );
 
         $this->assertCount(2, $constraint);
