@@ -456,11 +456,11 @@ final class TestSuiteSorterTest extends TestCase
                     'testWithProvider with data set "@fail2"' => ['state' => TestStatus::success(), 'time' => 1],
                 ],
                 [
-                    FaillingDataProviderTest::class . '::testWithProvider with data set "@good1"',
-                    FaillingDataProviderTest::class . '::testWithProvider with data set "@good2"',
-                    FaillingDataProviderTest::class . '::testWithProvider with data set "@fail1"',
-                    FaillingDataProviderTest::class . '::testWithProvider with data set "@good3"',
-                    FaillingDataProviderTest::class . '::testWithProvider with data set "@fail2"',
+                    FaillingDataProviderTest::class . '::testWithProvider with data set "good1"',
+                    FaillingDataProviderTest::class . '::testWithProvider with data set "good2"',
+                    FaillingDataProviderTest::class . '::testWithProvider with data set "fail1"',
+                    FaillingDataProviderTest::class . '::testWithProvider with data set "good3"',
+                    FaillingDataProviderTest::class . '::testWithProvider with data set "fail2"',
                     FaillingDataProviderTest::class . '::testOne',
                 ],
             ],
@@ -473,11 +473,11 @@ final class TestSuiteSorterTest extends TestCase
                     // empty result cache
                 ],
                 [
-                    FaillingDataProviderTest::class . '::testWithProvider with data set "@good1"',
-                    FaillingDataProviderTest::class . '::testWithProvider with data set "@good2"',
-                    FaillingDataProviderTest::class . '::testWithProvider with data set "@fail1"',
-                    FaillingDataProviderTest::class . '::testWithProvider with data set "@good3"',
-                    FaillingDataProviderTest::class . '::testWithProvider with data set "@fail2"',
+                    FaillingDataProviderTest::class . '::testWithProvider with data set "good1"',
+                    FaillingDataProviderTest::class . '::testWithProvider with data set "good2"',
+                    FaillingDataProviderTest::class . '::testWithProvider with data set "fail1"',
+                    FaillingDataProviderTest::class . '::testWithProvider with data set "good3"',
+                    FaillingDataProviderTest::class . '::testWithProvider with data set "fail2"',
                     FaillingDataProviderTest::class . '::testOne',
                 ],
             ],
@@ -486,19 +486,19 @@ final class TestSuiteSorterTest extends TestCase
                 TestSuiteSorter::ORDER_DEFAULT,
                 self::IGNORE_DEPENDENCIES,
                 [
-                    'testOne'                                 => ['state' => TestStatus::success(), 'time' => 1],
-                    'testWithProvider with data set "@good1"' => ['state' => TestStatus::success(), 'time' => 1],
-                    'testWithProvider with data set "@good2"' => ['state' => TestStatus::success(), 'time' => 1],
-                    'testWithProvider with data set "@good3"' => ['state' => TestStatus::success(), 'time' => 1],
-                    'testWithProvider with data set "@fail1"' => ['state' => TestStatus::error(), 'time' => 1],
-                    'testWithProvider with data set "@fail2"' => ['state' => TestStatus::error(), 'time' => 1],
+                    'testOne'                                => ['state' => TestStatus::success(), 'time' => 1],
+                    'testWithProvider with data set "good1"' => ['state' => TestStatus::success(), 'time' => 1],
+                    'testWithProvider with data set "good2"' => ['state' => TestStatus::success(), 'time' => 1],
+                    'testWithProvider with data set "good3"' => ['state' => TestStatus::success(), 'time' => 1],
+                    'testWithProvider with data set "fail1"' => ['state' => TestStatus::error(), 'time' => 1],
+                    'testWithProvider with data set "fail2"' => ['state' => TestStatus::error(), 'time' => 1],
                 ],
                 [
-                    FaillingDataProviderTest::class . '::testWithProvider with data set "@fail1"',
-                    FaillingDataProviderTest::class . '::testWithProvider with data set "@fail2"',
-                    FaillingDataProviderTest::class . '::testWithProvider with data set "@good1"',
-                    FaillingDataProviderTest::class . '::testWithProvider with data set "@good2"',
-                    FaillingDataProviderTest::class . '::testWithProvider with data set "@good3"',
+                    FaillingDataProviderTest::class . '::testWithProvider with data set "fail1"',
+                    FaillingDataProviderTest::class . '::testWithProvider with data set "fail2"',
+                    FaillingDataProviderTest::class . '::testWithProvider with data set "good1"',
+                    FaillingDataProviderTest::class . '::testWithProvider with data set "good2"',
+                    FaillingDataProviderTest::class . '::testWithProvider with data set "good3"',
                     FaillingDataProviderTest::class . '::testOne',
                 ],
             ],
